

# exp <- array(rnorm(1000), dim = c(dat = 1, lat = 3, lon = 5, member = 10, year = 50))
# ref <- array(rnorm(1000), dim = c(dat = 1, lat = 3, lon = 5, member = 10, year = 50))
# obs <- array(rnorm(1000), dim = c(dat = 1, lat = 3, lon = 5, year = 50))
# biasSS1 <- BiasSS(exp = exp, obs = obs, ref = ref, time_dim = 'year', memb_dim = 'member', na.rm = FALSE, ncores = 1)
# biasSS2 <- BiasSS(exp = exp, obs = obs, ref = NULL, time_dim = 'year', memb_dim = 'member', na.rm = FALSE, ncores = 1)

BiasSS <- function(exp, obs, ref = NULL, time_dim = 'sdate', memb_dim = NULL, na.rm = FALSE, ncores = NULL) {
  
  ## Checks
  
  
  ## Ensemble mean
  if (!is.null(memb_dim)) {
    exp <- MeanDims(exp, memb_dim, na.rm = na.rm)
    if (!is.null(ref)) {
      ref <- MeanDims(ref, memb_dim, na.rm = na.rm)
    }
  }
  
  ## Mean bias skill score
  if (is.null(ref)) {
    ss <- multiApply::Apply(data = list(exp, obs),
                            target_dims = time_dim,
                            fun = .BiasSS, na.rm = na.rm,
                            ref = ref, ncores = ncores)
    
  } else {
    ss <- multiApply::Apply(data = list(exp, obs, ref),
                            target_dims = time_dim,
                            fun = .BiasSS, na.rm = na.rm,
                            ncores = ncores)
  }
  return(ss)
}

.BiasSS <- function(exp, obs, ref = NULL, na.rm = FALSE) {
  
  if (isTRUE(na.rm)) {
    if (is.null(ref)) {
      good_values <- !is.na(exp) & !is.na(obs)
      exp <- exp[good_values]
      obs <- obs[good_values]
    } else {
      good_values <- !is.na(exp) & !is.na(ref) & !is.na(obs)
      exp <- exp[good_values]
      ref <- ref[good_values]
      obs <- obs[good_values]
    }
  }
  
  ## Bias of the exp
  bias_exp <- .Bias(exp = exp, obs = obs)
  
  ## Bias of the ref
  if (is.null(ref)) { ## Climatological forecast
    ref <- mean(obs, na.rm = na.rm)
  }
  bias_ref <- .Bias(exp = ref, obs = obs)
  
  ## Skill score and significance
  ss <- list()
  ss$biasSS <- 1 - mean(abs(bias_exp), na.rm = na.rm) / mean(abs(bias_ref), na.rm = na.rm)
  ss$sign <- .RandomWalkTest(skill_A = bias_exp, skill_B = bias_ref)$signif
  
  return(ss)
}
